
{-# OPTIONS -fffi #-}

module Common.MMap (mmap) where

import Data.Word (Word8)
import Foreign.C.Error (throwErrnoIf)
import Foreign.C.Types (CInt, CSize)
import Foreign.Concurrent (newForeignPtr)
import Foreign.ForeignPtr (ForeignPtr)
import Foreign.Ptr (Ptr, nullPtr, plusPtr)
import System.Posix.Files (getFdStatus, fileSize)
import System.Posix.IO (openFd, OpenMode(ReadOnly), defaultFileFlags, closeFd)
import System.Posix.Types (COff, Fd(Fd))

mmap :: FilePath -> IO (ForeignPtr Word8, Int)
mmap filename = do
    fd@(Fd c_fd) <- openFd filename ReadOnly Nothing defaultFileFlags
    stat <- getFdStatus fd
    let c_len = fromIntegral $ fileSize stat
    res <- throwErrnoIf (cMAP_FAILED ==) "mmap failed" $
           c_mmap nullPtr c_len cPROT_READ cMAP_SHARED c_fd 0
    closeFd fd
    fp <- newForeignPtr res (c_munmap res c_len >> return ())
    return (fp, fromIntegral $ fileSize stat)

#include <sys/mman.h>

cPROT_READ :: CInt
cPROT_READ = #const PROT_READ
cMAP_SHARED :: CInt
cMAP_SHARED = #const MAP_SHARED

cMAP_FAILED :: Ptr a
cMAP_FAILED = plusPtr nullPtr (-1) -- Eww eww eww

foreign import ccall unsafe "static sys/mman.h mmap"
    c_mmap :: Ptr a -> CSize -> CInt -> CInt -> CInt -> COff -> IO (Ptr b)
foreign import ccall unsafe "static sys/mman.h munmap"
    c_munmap :: Ptr a -> CSize -> IO CInt

