
module GenMakefile (main) where

import Data.List (intersperse)
import System.Environment (getArgs)

type TestNum = Int
type Command = String
type Module = String
type Input = String
data TestInfo = TestInfo {
                    desc :: String,
                    tests :: [Test],
                    gen_input :: Command,
                    clean_input :: Command,
                    inputs :: [Input],
                    comparisons :: [Comparison]
                }
    deriving Read
data Test = Test Module [TestAdditional]
    deriving Read
data TestAdditional = CModules [Module]
    deriving Read
data Comparison = Comparison Module Module
    deriving Read

main :: IO ()
main = do [gen_root, test_file] <- getArgs
          dir <- case splitAt (length test_file - 5) test_file of
                     (dir, "/test") -> return dir
                     _ -> error ("Bad argument: " ++ test_file)
          ti <- read_test_file (dir ++ "/test")
          let mf = dir ++ "/Makefile"
          writeFile mf ("clean::\n" ++
                        "\trm -f " ++ mf ++ "\n\n" ++
                        build dir ti)
          appendFile gen_root $ ("include " ++ mf ++ "\n")

build :: FilePath -> TestInfo -> String
build dir ti =
   -- Run the test
   (unlines $ map (build_test dir) $ tests ti) ++ "\n" ++
   "run_tests::\n" ++
   "\trm -f " ++ global_res_file ++ "\n" ++
   "\techo '<h2>" ++ dir ++ "</h2>' >> " ++ global_res_file ++ "\n" ++
   "\techo '<p>" ++ desc ti ++ "</p>' >> " ++ global_res_file ++ "\n" ++
   "\techo '<table border=\"1\"><tr><td></td>' >> "
                                    ++ global_res_file ++ "\n" ++
   "\techo " ++ concatMap (\(Comparison from to) ->
                           "'<th>" ++ mk_link from ++ " -&gt; "
                                   ++ mk_link to ++ "</th>'")
                          (comparisons ti)
                            ++ " >> " ++ global_res_file ++ "\n" ++
   "\techo '</tr>' >> " ++ global_res_file ++ "\n" ++
   unlines (concatMap do_inp (inputs ti)) ++ "\n" ++
   "\techo '</table>' >> " ++ global_res_file
   where mk_link what = "<a href=\"" ++dir ++ "/" ++ what ++ ".hs\">"
                     ++ what
                     ++ "</a>"
         do_inp inp = let inp' = dir ++ " " ++ inp
                      in -- Generate the test input
                         ["\t" ++ gen_input ti ++ " " ++ inp'] ++
                         map (do_test inp') (tests ti) ++
                         -- Clean the test input
                         ["\t" ++ clean_input ti ++ " " ++ inp'] ++
                         -- Now do the output
                         ["\techo '<tr>' >> " ++ global_res_file] ++
                         ["\techo '<td>" ++ inp ++ "</td>' >> "
                                      ++ global_res_file ++ "\n"] ++
                         map do_comp (comparisons ti) ++
                         ["\techo '</tr>' >> " ++ global_res_file]
         do_test inp' (Test m _) = "\t" ++ dir ++ "/" ++ m ++ " " ++ inp'
                                ++ " > " ++ mk_res_file m
         do_comp (Comparison from to)
          = "\t./GenResults " ++ mk_res_file from ++ " " ++ mk_res_file to
                              ++ " >> " ++ global_res_file
         mk_res_file m = dir ++ "/" ++ "/results_" ++ m
         global_res_file = dir ++ "/results"

build_test :: FilePath -> Test -> String
build_test dir (Test m ads)
 = -- How to build the test
   mod ++ ": " ++ extra_mods ++ "\n" ++
   "\t" ++ "$(GHC) $(GHC_OPTS) "
        ++ extra_mods
        ++ " --make $@ -o $@ -main-is " ++ mod_main ++ "\n\n" ++
   -- Clean the test
   "clean::\n"++
   "\trm -f " ++ mod ++ "\n\n" ++
   -- Add the test to PROGRAMS
   "PROGRAMS += " ++ mod ++ "\n\n"
    where get_extra_mods [] = []
          get_extra_mods (CModules ms:ads') = ms ++ get_extra_mods ads'
          mod = dir ++ "/" ++ m
          mod_main = map (\c -> if c == '/' then '.' else c) mod ++ ".main"
          extra_mods = concat $ intersperse " "
                     $ map (\x -> dir ++ "/" ++ x ++ ".o")
                     $ get_extra_mods ads

read_test_file :: FilePath -> IO TestInfo
read_test_file test_file = do xs <- readFile test_file
                              return (read xs)

