
module GenResults (main) where

import Control.Monad (when)
import Data.Ratio ((%))
import Numeric (showFFloat)
import System.Environment (getArgs)
import Text.Regex (mkRegex, matchRegex)

main :: IO ()
main = do [from, to] <- getArgs
          from' <- readFile from
          to' <- readFile to
          putStrLn $ compare_times (parse_line from') (parse_line to')

tolerance :: Rational
tolerance = 1.1

compare_times :: (Integer, Integer) -> (Integer, Integer) -> String
compare_times (count_from, time_from) (count_to, time_to)
 = "<td class=\"" ++ css_class ++ "\">" ++
   showFFloat (Just 2) (fromRational ratio :: Double) "" ++
   "</td>"
    where ratio = (time_to % count_to) / (time_from % count_from)
          css_class = if ratio > tolerance then "failed"
                      else if 1 / ratio > tolerance then "toofast"
                      else "passed"

parse_line :: String -> (Integer, Integer)
parse_line s = case matchRegex re s of
                   Just [num, time] -> (read num, read time)
                   _ -> error "Parsing failed"
    where re = mkRegex "^[a-zA-Z0-9_]+: x([0-9]+); time = ([0-9]+)ps; "

