
module Utils.Timer (time, timeIO) where

import System.CPUTime (getCPUTime)

time :: String -> (a -> b) -> a -> IO ()
time s function argument = iter 1
    where iter :: Int -> IO ()
          iter n = do start <- getCPUTime
                      seqList (map ($ argument) (replicate n function)) $ do
                          end <- getCPUTime
                          let gap = end - start
                              avg = picoToMilli (fromIntegral gap
                                               / fromIntegral n)
                          if gap < 10^(12 :: Int)
                            then iter (2*n)
                            else putStrLn (s ++ ": x" ++ show n
                                        ++ "; avg = " ++ show avg ++ "ms")
          seqList :: [a] -> b -> b
          seqList [] y = y
          seqList (x:xs) y = seq x $ seqList xs y

timeIO :: String -> IO a -> IO ()
timeIO s f = grow 1
    where grow :: Int -> IO ()
          grow n = do start <- getCPUTime
                      iter n
                      end <- getCPUTime
                      let gap = end - start
                          avg = picoToMilli (fromIntegral gap / fromIntegral n)
                          avg' = fromRational avg :: Double
                      if gap < 10^(12 :: Int)
                        then grow (2*n)
                        else putStrLn (s ++ ": x" ++ show n
                                    ++ "; time = " ++ show gap ++ "ps"
                                    ++ "; avg = " ++ show avg' ++ "ms")
          iter 0 = return ()
          iter (n+1) = f >> iter n

picoToMilli :: Rational -> Rational
picoToMilli n = n / 10^(9 :: Int)

